"use strict";
const EXPORTED_SYMBOLS = ["application"];
const {
    classes: Cc,
    interfaces: Ci,
    utils: Cu
} = Components;
Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");
const application = {
    get Lib() {
        return this._libs;
    },
    get brandID() {
        return this._getBrandId();
    },
    get bannerId() {
        return this._modules.bannerId.id;
    },
    get id() {
        return this._core.extensionId;
    },
    get version() {
        return this._core.extensionVersion;
    },
    get locale() {
        if (!this._locale) {
            this._locale = this._getLocale();
        }
        return this._locale;
    },
    get extensionsDataDir() {
        if (!this._extensionsDataDir) {
            this._extensionsDataDir = Services.dirsvc.get("ProfD", Ci.nsIFile);
            this._extensionsDataDir.append("yandex-extensions-data");
            if (!this._extensionsDataDir.exists()) {
                this._extensionsDataDir.create(Ci.nsIFile.DIRECTORY_TYPE, parseInt("0755", 8));
            }
        }
        return this._extensionsDataDir.clone();
    },
    get extensionDataDir() {
        let extensionDataDir = this.extensionsDataDir;
        extensionDataDir.append(this.id);
        if (!extensionDataDir.exists()) {
            extensionDataDir.create(Ci.nsIFile.DIRECTORY_TYPE, parseInt("0755", 8));
        }
        return extensionDataDir;
    },
    get yCookieData() {
        return {
            fieldName: this._buildConfig.yasoft,
            fieldValue: this.version
        };
    },
    get isNewStart() {
        return this._newStart;
    },
    get isUpdate() {
        return this._updateStart;
    },
    start: function (core, startInfo) {
        this._core = core;
        this._newStart = Boolean(startInfo.newStart);
        this._updateStart = Boolean(startInfo.updateStart);
        this._defineGetters();
        this._loadLibraries();
        this._migratePreferences();
        this._startModules();
    },
    rollback: function (isUpdate) {
        this._stopModules(isUpdate);
        this._libs = null;
        this._core = null;
    },
    getClid: function (id) {
        let brandedClidSet = this._buildConfig["clidSet"][this.brandID] || Object.create(null);
        return brandedClidSet["clid" + id];
    },
    getContentPath: function () {
        return this._core.appContentPath;
    },
    getLocalePath: function () {
        return this._core.appLocalePath;
    },
    get preferences() {
        let appPrefs = new this.Lib.Preferences("extensions." + this.id + ".");
        delete this.preferences;
        Object.defineProperty(this, "preferences", {
            enumerable: true,
            value: appPrefs
        });
        return appPrefs;
    },
    _brandId: null,
    _locale: null,
    _newStart: false,
    _libraryFileNames: [
        "Preferences.jsm",
        "Foundation.jsm",
        "DataURI.jsm",
        "WinReg.jsm"
    ],
    get _moduleFileNames() {
        return this._modulesConfig;
    },
    _defineGetters: function () {
        XPCOMUtils.defineLazyGetter(this, "_buildConfig", () => {
            try {
                return this.Lib.fileutils.jsonFromFile(this._core.buildConfigFile);
            } catch (e) {
                return Object.create(null);
            }
        });
        XPCOMUtils.defineLazyGetter(this, "_modulesConfig", () => {
            let stream = this._core.getFileInputStream(this.getContentPath() + "modules.json");
            return this.Lib.fileutils.jsonFromStream(stream);
        });
    },
    _migratePreferences: function () {
        let Preferences = this.Lib.Preferences;
        let oldPrefBranch = this.id + ".";
        let newPrefBranch = "extensions." + oldPrefBranch;
        Services.prefs.getBranch(oldPrefBranch).getChildList("", {}).forEach(prefKey => {
            let prefValue = Preferences.get(oldPrefBranch + prefKey, null);
            if (prefValue !== null) {
                Preferences.set(newPrefBranch + prefKey, prefValue);
            }
        });
        Preferences.resetBranch(oldPrefBranch);
    },
    _loadLibraries: function () {
        this._libs = this._importFilesContent(this._getLibrariesPath(), this._libraryFileNames);
    },
    _getLibrariesPath: function () {
        return this.getContentPath() + "libraries/";
    },
    _startModules: function () {
        this._modules = this._importFilesContent(this._getModulesPath(), this._moduleFileNames);
        this._iterateCollectionAndCallMethod(this._modules, ["init"], [this]);
    },
    _getModulesPath: function () {
        return this.getContentPath() + "modules/";
    },
    _importFilesContent: function (pathToFiles, filenames) {
        return filenames.reduce((importsObject, filename) => {
            Cu.unload(pathToFiles + filename);
            Cu.import(pathToFiles + filename, importsObject);
            return importsObject;
        }, Object.create(null));
    },
    _stopModules: function (isUpdate) {
        let methods = ["finalize"];
        if (!isUpdate) {
            methods.unshift("rollback");
        }
        this._iterateCollectionAndCallMethod(this._modules, methods);
        this._modules = null;
    },
    _iterateCollectionAndCallMethod: function (collection, methodNames, methodArguments) {
        let collectionKeys = Object.keys(collection);
        if (methodNames.some(name => [
                "finalize",
                "rollback"
            ].indexOf(name) > -1)) {
            collectionKeys.reverse();
        }
        collectionKeys.forEach(collectionKey => {
            let collectionObject = collection[collectionKey];
            methodNames.forEach(methodName => {
                if (typeof collectionObject[methodName] === "function") {
                    collectionObject[methodName].apply(collectionObject, methodArguments);
                }
            });
        });
    },
    _getLocale: function () {
        let xulChromeReg = Cc["@mozilla.org/chrome/chrome-registry;1"].getService(Ci.nsIXULChromeRegistry);
        let localeString = "en";
        try {
            localeString = xulChromeReg.getSelectedLocale(this._core.appName);
        } catch (e) {
            localeString = xulChromeReg.getSelectedLocale("global");
        }
        return this._parseLocaleString(localeString);
    },
    _parseLocaleString: function (localeString) {
        let components = localeString.match(this._localePattern);
        if (components) {
            return {
                language: components[1],
                country: components[3],
                region: components[5]
            };
        }
        return null;
    },
    _localePattern: /^([a-z]{2})(-([A-Z]{2})(-(\w{2,5}))?)?$/,
    _getBrandId: function () {
        if (!this._brandId) {
            let brandId = this.preferences.get("branding", null);
            if (!brandId) {
                brandId = this._modules.brander.SUGGESTED_BRAND_ID;
                this.preferences.set("branding", brandId);
            }
            this._brandId = brandId;
        }
        return this._brandId;
    }
};