"use strict";
const EXPORTED_SYMBOLS = ["addonStatus"];
const {
    classes: Cc,
    interfaces: Ci,
    utils: Cu
} = Components;
Cu.import("resource://gre/modules/Services.jsm");
const DAY_START_HOUR = 3;
const PREFS = {
    firstStart: "status.firstStart",
    lastCollect: "status.lastCollect",
    ui: "status.guid.value"
};
const addonStatus = {
    init: function (application) {
        this._application = application;
        this._migrateOldPreferences();
        this._getFirstStartTime();
        this._scheduleStatCollector();
    },
    rollback: function () {
        this._prepareUninstallData();
        this._resetPrefs();
    },
    finalize: function () {
        this._stopStatCollector();
        this._application = null;
        this._uiString = null;
    },
    get ui() {
        if (this._uiString !== null) {
            return this._uiString;
        }
        let ui = "";
        let prefName = PREFS.ui;
        let uiFile = this._uiFile;
        if (uiFile.exists()) {
            try {
                ui = this._getUiFromString(this._application.Lib.fileutils.readTextFile(uiFile));
            } catch (e) {
            }
        } else {
            if (this._application.preferences.has(prefName)) {
                ui = this._getUiFromString(this._application.preferences.get(prefName, ""));
            } else {
                ui = Cc["@mozilla.org/uuid-generator;1"].getService(Ci.nsIUUIDGenerator).generateUUID().toString();
            }
            if (ui) {
                try {
                    this._application.Lib.fileutils.writeTextFile(uiFile, ui);
                } catch (e) {
                }
            }
        }
        if (ui) {
            this._application.preferences.set(prefName, ui);
        }
        return this._uiString = ui || "";
    },
    get firstStartTime() {
        return this._getFirstStartTime();
    },
    get lastCollectTime() {
        return this._lastCollectTime;
    },
    _getCollectInterval: function () {
        let dateNow = new Date();
        let day = dateNow.getDate() - (dateNow.getHours() < DAY_START_HOUR ? 1 : 0);
        let dateStart = new Date(dateNow.getFullYear(), dateNow.getMonth(), day, DAY_START_HOUR);
        let dateStartTime = Math.floor(dateStart.valueOf() / 1000);
        let collectInterval = 0;
        const oneDaySec = 24 * 60 * 60;
        let timeDiff = dateStartTime - this._lastCollectTime;
        if (Math.abs(timeDiff) >= oneDaySec) {
            collectInterval = 0;
        } else if (timeDiff > 0) {
            collectInterval = 0;
        } else {
            collectInterval = oneDaySec - Math.floor((dateNow.valueOf() - dateStart.valueOf()) / 1000);
        }
        collectInterval *= 1000;
        return collectInterval;
    },
    _collectorMinDelayMS: 60000,
    get _lastCollectTime() {
        let collectTimePrefValue = this._application.preferences.get(PREFS.lastCollect, 0);
        collectTimePrefValue = parseInt(collectTimePrefValue, 10) || 0;
        return Math.floor(collectTimePrefValue / 1000);
    },
    set _lastCollectTime(timestamp) {
        this._application.preferences.set(PREFS.lastCollect, String(timestamp));
    },
    _getFirstStartTime: function () {
        let prefName = PREFS.firstStart;
        let firstStart = this._application.preferences.get(prefName, 0);
        if (!firstStart) {
            firstStart = this._lastCollectTime || Date.now();
            this._application.preferences.set(prefName, String(firstStart));
        }
        firstStart = parseInt(firstStart, 10) || 0;
        delete this._getFirstStartTime;
        this._getFirstStartTime = () => firstStart;
        return this._getFirstStartTime();
    },
    _uiString: null,
    _getUiFromString: function (ui) {
        ui = ui && ui.toString();
        return ui && /^{[0-9a-f]{2,8}(\-[0-9a-f]{2,4}){2}\-[0-9a-f]{1,4}\-[0-9a-f]{2,12}}$/i.test(ui) ? ui : null;
    },
    get _uiFile() {
        let uiFile = this._application.extensionsDataDir;
        uiFile.append("ui");
        if (!uiFile.exists()) {
            let uiOldFile = this._userDir;
            uiOldFile.append("ui");
            if (uiOldFile.exists()) {
                try {
                    uiOldFile.copyTo(uiFile.parent, uiFile.leafName);
                    uiFile.permissions = parseInt("0644", 8);
                } catch (e) {
                }
            }
        }
        return uiFile;
    },
    get _userDir() {
        let isWindowsOS = /^win/i.test(Services.appinfo.OS);
        let userDir = Services.dirsvc.get(isWindowsOS ? "AppData" : "Home", Ci.nsIFile);
        userDir.append(isWindowsOS ? "Yandex" : ".yandex");
        Object.defineProperty(this, "userDir", {
            enumerable: true,
            get: () => userDir.clone()
        });
        return this.userDir;
    },
    _scheduleStatCollector: function () {
        if (this._application.isNewStart) {
            this._collect({ stat: "install" });
        } else {
            this._armCollectTimer();
        }
    },
    _stopStatCollector: function () {
        this._stopTimer("_collectTimer");
    },
    _armCollectTimer: function () {
        let delay = this._getCollectInterval();
        this._armTimerWithMinDelay(delay, "_collectTimer", "_collect");
    },
    _armTimerWithMinDelay: function (delay, timerName, callbackName) {
        if ((delay || 0) < this._collectorMinDelayMS) {
            delay = this._collectorMinDelayMS;
        }
        this._createTimer(timerName, () => this[callbackName](), delay);
    },
    _createTimer: function (timerName, callback, delay) {
        this._stopTimer(timerName);
        this[timerName] = new this._application.Lib.sysutils.Timer(callback, delay);
    },
    _stopTimer: function (timerName) {
        if (this[timerName]) {
            this[timerName].cancel();
            this[timerName] = null;
        }
    },
    _collect: function (customData) {
        this._lastCollectTime = Date.now();
        this._prepareIntervalData(customData);
        this._prepareElduseData();
        this._armCollectTimer();
    },
    _prepareIntervalData: function (customData) {
        let {urlDataDelivery} = this._application._modules;
        urlDataDelivery.addRecord({
            provider: "addonStatus",
            url: this._createStatusURL(customData),
            stat: true
        });
    },
    _prepareElduseData: function () {
        let {urlDataDelivery} = this._application._modules;
        urlDataDelivery.addRecord({
            provider: "addonStatus",
            url: this._createElduseURL(),
            stat: true
        });
    },
    _createStatusURL: function (customData) {
        return "https://soft.export.yandex.ru/status.xml?" + this._createStatQuery(customData);
    },
    _createStatQuery: function (customData) {
        let statData = this._statData;
        if (customData) {
            for (let [propName, propValue] in Iterator(customData)) {
                statData[propName] = propValue;
            }
        }
        return Object.keys(statData).map(key => {
            return key + "=" + encodeURIComponent(statData[key]);
        }).join("&");
    },
    _createElduseURL: function () {
        let host = "yandex.ru";
        switch (this._application.brandID) {
        case "ua":
            host = "yandex.ua";
            break;
        case "tb":
            host = "yandex.com.tr";
            break;
        }
        let url = "https://" + host + "/clck/click/dtype=elduse/path=tech.yaelements.dayuse/vars=";
        let vars = [];
        let elduseVars = this._elduseVars;
        for (let key of Object.keys(elduseVars)) {
            vars.push("-" + key + "=" + encodeURIComponent(elduseVars[key]));
        }
        url += vars.join(",");
        url += "/slots=0,0,0/*";
        return url;
    },
    get _elduseVars() {
        let vars = {
            dayuse: this._getDayuseDays(),
            clid1: this._application.getClid(1),
            bro: "firefox",
            productname: this._getYasoftValue(),
            ver: this._application.version.replace(/\./g, "-"),
            ui: this.ui
        };
        let bannerId = this._application.bannerId;
        if (bannerId) {
            vars.bnrd = bannerId;
        }
        return vars;
    },
    _getDayuseDays: function () {
        let firstStart = this.firstStartTime;
        if (!firstStart || typeof firstStart !== "number" || firstStart < 0) {
            return null;
        }
        function getEndOfDayTime(timestamp) {
            let now = new Date(timestamp);
            let dateNumber = now.getHours() < DAY_START_HOUR ? now.getDate() - 1 : now.getDate();
            return new Date(now.getFullYear(), now.getMonth(), dateNumber, DAY_START_HOUR).valueOf();
        }
        let endOfInstallDayTime = getEndOfDayTime(firstStart);
        let ageInDays = Math.floor((Date.now() - endOfInstallDayTime) / 86400000);
        if (ageInDays < 0) {
            return null;
        }
        return ageInDays;
    },
    _prepareUninstallData: function () {
        let {urlDataDelivery} = this._application._modules;
        urlDataDelivery.addRecord({
            provider: "addonStatus",
            url: this._createStatusURL({ stat: "uninstall" }),
            stat: true,
            priority: urlDataDelivery.PRIORITY.HIGH
        }, true);
    },
    _resetPrefs: function () {
        Object.keys(PREFS).forEach(prefName => {
            this._application.preferences.reset(PREFS[prefName]);
        });
    },
    get _statData() {
        let appInfo = Services.appinfo;
        let statistics = {
            brandID: this._application.brandID,
            bn: appInfo.name,
            bv: appInfo.version,
            clid: this._application.getClid(1),
            lang: this._application.locale.language,
            os: appInfo.OS,
            stat: "dayuse",
            ui: this.ui,
            ver: this._application.version,
            yasoft: this._getYasoftValue(),
            tl: "{timeLastSend}",
            fd: "{timeFirstSend}"
        };
        let bannerId = this._application.bannerId;
        if (bannerId) {
            statistics.bnrd = bannerId;
        }
        return statistics;
    },
    _getYasoftValue: function () {
        return this._application.yCookieData.fieldName;
    },
    _migrateOldPreferences: function () {
        let lastSuccesSent = this._application.preferences.get("status.lastSuccess", 0);
        lastSuccesSent = parseInt(lastSuccesSent, 10) || 0;
        let firstSuccesSent = this._application.preferences.get("status.firstSuccess", 0);
        firstSuccesSent = parseInt(firstSuccesSent, 10) || 0;
        if (firstSuccesSent && /^(\d{4})\.(\d{2})\.(\d{2})$/.test(firstSuccesSent)) {
            firstSuccesSent = new Date(parseInt(RegExp.$1, 10), parseInt(RegExp.$2, 10) - 1, parseInt(RegExp.$3, 10)).valueOf() || 0;
        } else {
            firstSuccesSent = 0;
        }
        if (!lastSuccesSent && !firstSuccesSent) {
            return;
        }
        let {urlDataDelivery} = this._application._modules;
        urlDataDelivery.updateProviderSendTime("addonStatus", {
            timeLastSend: lastSuccesSent,
            timeFirstSend: firstSuccesSent
        });
        this._application.preferences.reset("status.lastSuccess");
        this._application.preferences.reset("status.firstSuccess");
    }
};
function isErrorRequest(request) {
    return !request || request.type === "error" || !request.target || request.target.status < 200 || request.target.status > 299;
}