"use strict";
const EXPORTED_SYMBOLS = ["urlDataDelivery"];
const {
    classes: Cc,
    interfaces: Ci,
    utils: Cu
} = Components;
let formatDate;
let Timer;
let isStopped = false;
const urlDataDelivery = {
    PRIORITY: {
        HIGH: 10,
        DEFAULT: 5,
        LOW: 0
    },
    init: function (application) {
        isStopped = false;
        this._application = application;
        formatDate = this._application.Lib.strutils.formatDate;
        Timer = this._application.Lib.sysutils.Timer;
        this._startTimers();
    },
    finalize: function () {
        isStopped = true;
        this._stopTimers();
    },
    addRecord: function (record, sendImmediately = false) {
        let url = record.url;
        if (!url || !/^https?:\/\//.test(url)) {
            throw new RangeError("Wrong url value (" + url + ")");
        }
        let priority;
        if ("priority" in record) {
            for (let key of Object.keys(this.PRIORITY)) {
                if (record.priority === this.PRIORITY[key]) {
                    priority = this.PRIORITY[key];
                    break;
                }
            }
        } else {
            priority = this.PRIORITY.DEFAULT;
        }
        if (typeof priority === "undefined") {
            throw new RangeError("Wrong priority value (" + record.priority + ")");
        }
        let provider = record.provider || "unknown";
        let postData = record.postData || null;
        let stat = typeof record.stat === "boolean" ? record.stat : true;
        let timeCreated = record.timeCreated || Date.now();
        let timeExpired = record.timeExpired || Date.now() + this._consts.EXPIRY_TIME;
        let maxSendAttempts = record.maxSendAttempts || this._consts.MAX_SEND_ATTEMPTS;
        if (!sendImmediately) {
            this._database.executeQueryAsync({
                query: "INSERT INTO queries " + "(provider, url, postData, stat, timeCreated, timeExpired, sendAttempts, maxSendAttempts, priority) " + "VALUES (:provider, :url, :postData, :stat, :timeCreated, :timeExpired, :sendAttempts, :maxSendAttempts, :priority)",
                parameters: {
                    provider: provider,
                    url: url,
                    postData: postData,
                    stat: stat,
                    timeCreated: timeCreated,
                    timeExpired: timeExpired,
                    sendAttempts: 0,
                    maxSendAttempts: maxSendAttempts,
                    priority: priority
                },
                callback: () => {
                    if (!isStopped) {
                        this._trySendNextRecord();
                    }
                }
            });
        } else {
            let addonStatus = this._application._modules.addonStatus;
            let times = {
                timeLastSend: this._lastSendTime || addonStatus.lastCollectTime,
                timeFirstSend: formatDate(new Date(this._firstSendTime || addonStatus.firstStartTime), "%Y.%M.%D")
            };
            this._sendRequest(null, this._replaceUrlTimeParams(url, times), postData);
        }
    },
    updateProviderSendTime: function (providerName, times, callback) {
        this._firstSendTime = times.timeFirstSend;
        this._lastSendTime = parseInt(times.timeLastSend / 1000, 10) || 0;
        this._database.executeQueryAsync({
            query: "INSERT OR REPLACE INTO providers " + "(id, timeLastSend, timeFirstSend) " + "VALUES (:id, :timeLastSend, :timeFirstSend)",
            parameters: {
                id: providerName,
                timeLastSend: times.timeLastSend || Date.now(),
                timeFirstSend: times.timeFirstSend || Date.now()
            },
            callback: () => {
                if (callback) {
                    callback();
                }
            }
        });
    },
    _firstSendTime: null,
    _lastSendTime: null,
    _startTimers: function () {
        this._timers = new Map();
        this._setRequestTimer();
    },
    _stopTimers: function () {
        for (let [timerName, timer] of this._timers) {
            timer.cancel();
        }
        this._timers = null;
    },
    _createTimer: function (timerName, callback, delay) {
        this._cancelTimer(timerName);
        if (this._timers) {
            let timer = new Timer(callback, delay);
            this._timers.set(timerName, timer);
        }
    },
    _cancelTimer: function (timerName) {
        if (this._timers && this._timers.has(timerName)) {
            this._timers.get(timerName).cancel();
            this._timers.delete(timerName);
        }
    },
    _setRequestTimer: function () {
        this._createTimer("undelivered", () => this._trySendNextRecord(), this._consts.SEND_INTERVAL_MS);
    },
    _isRequestSending: false,
    _trySendNextRecord: function () {
        if (this._isRequestSending) {
            return;
        }
        this._isRequestSending = true;
        this._getRecordForSend((id, url, postData) => {
            if (typeof id === "undefined") {
                this._isRequestSending = false;
                return;
            }
            if (!isStopped) {
                this._sendRequest(id, url, postData);
            }
        });
    },
    _getRecordForSend: function (callback) {
        this._cleanupQueries();
        this._database.executeQueryAsync({
            query: "SELECT id, provider, url, postData FROM queries ORDER BY priority DESC, sendAttempts ASC LIMIT 1",
            columns: [
                "id",
                "provider",
                "url",
                "postData"
            ],
            callback: (results, error) => {
                if (error) {
                    Cu.reportError(error);
                    callback();
                    return;
                }
                let {id, provider, url, postData} = results[0] || {};
                if (!provider) {
                    callback();
                    return;
                }
                this._getProviderSendTime(provider, times => {
                    this._firstSendTime = times.timeFirstSend;
                    this._lastSendTime = times.timeLastSend;
                    callback(id, this._replaceUrlTimeParams(url, times), postData);
                });
            }
        });
    },
    _replaceUrlTimeParams: function (url, times) {
        return url.replace(/(\{|%7B)timeLastSend(\}|%7D)/g, times.timeLastSend).replace(/(\{|%7B)timeFirstSend(\}|%7D)/g, times.timeFirstSend);
    },
    _sendRequest: function (id, url, postData) {
        let request = this._createXmlHttpRequest(url, postData);
        let callbackFunc = this._createCallbackFunction(id);
        request.QueryInterface(Ci.nsIDOMEventTarget);
        request.addEventListener("load", callbackFunc, false);
        request.addEventListener("error", callbackFunc, false);
        request.send(postData || null);
    },
    _createXmlHttpRequest: function (url, postData) {
        let request = Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(Ci.nsIXMLHttpRequest);
        request.mozBackgroundRequest = true;
        if (postData) {
            request.open("POST", url, true);
            request.setRequestHeader("Content-Type", "application/x-www-form-urlencoded; charset=UTF-8");
        } else {
            request.open("GET", url, true);
        }
        request.channel.loadFlags |= Ci.nsIRequest.LOAD_BYPASS_CACHE;
        request.setRequestHeader("Cache-Control", "no-cache");
        request.setRequestHeader("Connection", "close");
        return request;
    },
    _createCallbackFunction: function (id) {
        if (id === null) {
            return () => {
            };
        }
        return this._onResponse.bind(this, id);
    },
    _onResponse: function (id, request) {
        this._isRequestSending = false;
        if (isErrorRequest(request)) {
            this._increaseQuerySendAttempts(id);
            this._setRequestTimer();
        } else {
            this._updateSendTime(id, () => {
                this._deleteQuery(id);
                this._trySendNextRecord();
            });
        }
    },
    _cleanupQueries: function () {
        let dropStatQueries = !this._sendUsageStat;
        this._database.executeQueryAsync({
            query: "DELETE FROM queries " + "WHERE (" + "timeExpired <= :timeNow " + "OR sendAttempts >= maxSendAttempts " + (dropStatQueries ? "OR stat = 1 " : "") + ")",
            parameters: { timeNow: Date.now() }
        });
    },
    _increaseQuerySendAttempts: function (queryId) {
        this._database.executeQueryAsync({
            query: "UPDATE queries SET sendAttempts = sendAttempts + 1 WHERE id = :id",
            parameters: { id: queryId }
        });
    },
    _deleteQuery: function (queryId) {
        this._database.executeQueryAsync({
            query: "DELETE FROM queries WHERE id = :id",
            parameters: { id: queryId }
        });
    },
    _updateSendTime: function (queryId, callback) {
        this._database.executeQueryAsync({
            query: "SELECT provider FROM queries WHERE id = :id",
            parameters: { id: queryId },
            columns: ["provider"],
            callback: (results, error) => {
                if (error) {
                    Cu.reportError(error);
                    callback();
                    return;
                }
                let providerName = results[0] && results[0].provider || null;
                if (!providerName) {
                    Cu.reportError("Can not get provider for query with id = " + queryId);
                    callback();
                    return;
                }
                this._database.executeQueryAsync({
                    query: "SELECT timeLastSend, timeFirstSend FROM providers WHERE id = :providerName",
                    columns: [
                        "timeLastSend",
                        "timeFirstSend"
                    ],
                    parameters: { providerName: providerName },
                    callback: (results, error) => {
                        let {timeLastSend, timeFirstSend} = !error && results[0] || {};
                        this.updateProviderSendTime(providerName, {
                            timeLastSend: Date.now(),
                            timeFirstSend: timeFirstSend || Date.now()
                        }, callback);
                    }
                });
            }
        });
    },
    _getProviderSendTime: function (providerName, callback) {
        this._database.executeQueryAsync({
            query: "SELECT timeLastSend, timeFirstSend FROM providers WHERE id = :providerName",
            columns: [
                "timeLastSend",
                "timeFirstSend"
            ],
            parameters: { providerName: providerName },
            callback: (results, error) => {
                let result = !error && results[0] || {};
                result.timeLastSend = parseInt(result.timeLastSend / 1000, 10) || 0;
                result.timeFirstSend = new Date(result.timeFirstSend || Date.now());
                result.timeFirstSend = formatDate(result.timeFirstSend, "%Y.%M.%D");
                callback(result);
            }
        });
    },
    __sendUsageStat: true,
    get _sendUsageStat() {
        if (this._application.statistics) {
            this.__sendUsageStat = this._application.statistics.sendUsageStat;
        }
        return this.__sendUsageStat;
    },
    __database: null,
    get _database() {
        if (!this.__database) {
            this.__database = new this._application.Lib.Database(this._dbFile, this._consts.INIT_QUERIES_TABLE_QUERIES);
        }
        return this.__database;
    },
    set _database(val) {
        if (val !== null) {
            throw new Error("Database setter accept only null value");
        }
        if (this.__database) {
            this.__database.close();
        }
        this.__database = null;
    },
    get _dbFile() {
        let dbFile = this._application.extensionDataDir;
        dbFile.append("urlDataDelivery.sqlite");
        return dbFile;
    },
    _consts: {
        SEND_INTERVAL_MS: 60 * 3 * 1000,
        MAX_SEND_ATTEMPTS: 5,
        EXPIRY_TIME: 60 * 60 * 24 * 5 * 1000,
        INIT_QUERIES_TABLE_QUERIES: [
            "CREATE TABLE IF NOT EXISTS queries (" + " id INTEGER PRIMARY KEY," + " provider TEXT NOT NULL," + " url BLOB NOT NULL," + " postData BLOB," + " stat BOOLEAN NOT NULL," + " timeCreated INTEGER," + " timeExpired INTEGER," + " sendAttempts INTEGER," + " maxSendAttempts INTEGER," + " priority INTEGER" + " );",
            "CREATE TABLE IF NOT EXISTS providers (" + " id TEXT PRIMARY KEY," + " timeLastSend INTEGER," + " timeFirstSend INTEGER" + " );"
        ]
    }
};
function isErrorRequest(request) {
    return !request || request.type === "error" || !request.target || request.target.status < 200 || request.target.status > 299;
}