"use strict";
const EXPORTED_SYMBOLS = ["yCookie"];
const {
    classes: Cc,
    interfaces: Ci,
    utils: Cu
} = Components;
Cu.import("resource://gre/modules/Services.jsm");
const COOKIE_MAX_EXPIRY = Math.pow(2, 62);
const TRUSTED_DOMAINS = [
    ".yandex.ru",
    ".yandex.kz",
    ".yandex.ua",
    ".yandex.by",
    ".yandex.net",
    ".yandex.com",
    ".yandex.com.tr",
    ".ya.ru"
];
const DECLINED_DOMAINS = [".moikrug.ru"];
let Lib;
const yCookie = {
    get DEFAULT_DOMAIN() {
        return ".yandex.ru";
    },
    get app() {
        return this._app;
    },
    get sessionCookieName() {
        return "ys";
    },
    get permanentCookieName() {
        return "yp";
    },
    init: function (application) {
        this._app = application;
        Lib = application.Lib;
        this._setSessionCookie();
        this._cleanDeclinedDomainsSessionCookie();
    },
    rollback: function () {
        this.setSessionCookieFieldValue(this._prepareExtSessionCookieField().name, null);
    },
    _setSessionCookie: function () {
        let field = this._prepareExtSessionCookieField();
        this.setSessionCookieFieldValue(field.name, field.value);
    },
    _prepareExtSessionCookieField: function () {
        return {
            name: this.app.yCookieData.fieldName,
            value: encodeURIComponent(this.app.yCookieData.fieldValue.replace(/\./g, "-"))
        };
    },
    _cleanDeclinedDomainsSessionCookie: function () {
        for (let domain of DECLINED_DOMAINS) {
            removeCoookie({
                domain: domain,
                cookieName: this.sessionCookieName
            });
        }
    },
    getSessionCookieFieldValue: function (fieldName, domain) {
        let sessionCookieValue = this._getCookieValue(this.sessionCookieName, domain || this.DEFAULT_DOMAIN);
        if (sessionCookieValue) {
            return this._getSessionCookieFieldValue(sessionCookieValue, fieldName);
        }
        return null;
    },
    _getSessionCookieFieldValue: function (cookieValue, fieldName) {
        let reg = new RegExp("(?:^|#)" + Lib.strutils.escapeRE(fieldName) + "\\.([^#]+)");
        let match = cookieValue.match(reg);
        return match && decodeURIComponent(match[1]);
    },
    getPermanentCookieFieldValue: function (fieldName, domain) {
        let permanentCookieValue = this._getCookieValue(this.permanentCookieName, domain || this.DEFAULT_DOMAIN);
        if (permanentCookieValue) {
            return this._getPermanentCookieFieldValue(permanentCookieValue, fieldName);
        }
        return null;
    },
    _getPermanentCookieFieldValue: function (cookieValue, fieldName) {
        let reg = new RegExp("(?:^|#)(?:\\d+)\\." + Lib.strutils.escapeRE(fieldName) + "\\.([^#]+)");
        let match = cookieValue.match(reg);
        return match && decodeURIComponent(match[1]);
    },
    _getCookieValue: function (cookieName, domain) {
        let timeNowSec = Math.ceil(Date.now() / 1000);
        let value = null;
        Lib.netutils.getCookiesFromHost(domain).some(cookie => {
            if (cookie.name === cookieName) {
                if (cookie.isSession || timeNowSec < cookie.expiry) {
                    value = cookie.value;
                    return true;
                }
            }
        });
        return value;
    },
    setSessionCookieFieldValue: function (fieldName, fieldValue) {
        this._setYCookieValue(this.sessionCookieName, fieldName, fieldValue);
    },
    setPermanentCookieFieldValue: function (fieldName, fieldValue, expire) {
        this._setYCookieValue(this.permanentCookieName, fieldName, fieldValue, expire);
    },
    _setYCookieValue: function (cookieName, fieldName, fieldValue, expire) {
        for (let domain of TRUSTED_DOMAINS) {
            let newCookieValue = this._injectFieldToCookieValue(this._getCookieValue(cookieName, domain), fieldName, fieldValue, expire);
            if (newCookieValue.length) {
                Services.cookies.add(domain, "/", cookieName, newCookieValue, false, false, true, COOKIE_MAX_EXPIRY);
            } else {
                removeCoookie({
                    domain: domain,
                    cookieName: cookieName
                });
            }
        }
    },
    _injectFieldToCookieValue: function (cookieValue, fieldName, fieldValue, fieldExpire) {
        let fields = this._extractFieldsFromCookieValue(cookieValue);
        let index = this._findFieldByName(fields, fieldName);
        let newField = this._prepareField(fieldName, fieldValue, fieldExpire);
        if (index !== null) {
            fields[index] = newField;
        } else {
            fields.push(newField);
        }
        return fields.filter(Boolean).join("#");
    },
    _extractFieldsFromCookieValue: function (cookieValue) {
        return (cookieValue || "").split("#");
    },
    _findFieldByName: function (fields, fieldName) {
        let index = null;
        fields.some((field, i) => {
            let r = new RegExp("^(?:\\d+\\.)?" + fieldName + "\\.", "i");
            if (r.test(field)) {
                index = i;
                return true;
            }
        });
        return index;
    },
    _prepareField: function (name, value, expires) {
        let field = null;
        if (value !== null) {
            field = name + "." + encodeURIComponent(value);
            if (expires) {
                field = expires + "." + field;
            }
        }
        return field;
    }
};
function removeCoookie({domain, cookieName, path, block}) {
    path = setDefaultArgumentValue(path, "/");
    block = setDefaultArgumentValue(block, false);
    Services.cookies.remove(domain, cookieName, path, block, {});
}
function setDefaultArgumentValue(argValue, defaultValue) {
    return typeof argValue === "undefined" ? defaultValue : argValue;
}